﻿using System.Threading.Tasks;
using Windows.Devices.Gpio;
using Windows.Foundation;
using Windows.UI.Core;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;

namespace TestApplication
{
    /// <summary>
    /// 메인 페이지
    /// </summary>
    public sealed partial class MainPage : Page
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// GPIO 컨트롤러
        /// </summary>
        private GpioController gpioController = null;

        /// <summary>
        /// GPIO 핀
        /// </summary>
        private GpioPin gpioPin = null;

        /// <summary>
        /// PIR 핀 번호
        /// </summary>
        private const int PIR_PIN_NUMBER = 5;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainPage()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainPage()
        {
            InitializeComponent();

            Loaded += Page_Loaded;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 페이지 로드시 처리하기 - Page_Loaded(sender, e)
 
        /// <summary>
        /// 페이지 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private async void Page_Loaded(object sender, RoutedEventArgs e)
        {
            await InitializeGPIO();
        }

        #endregion
        #region GPIO 핀 값 변경시 처리하기 - gpioPin_ValueChanged(sender, e)

        /// <summary>
        /// GPIO 핀 값 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void gpioPin_ValueChanged(GpioPin sender, GpioPinValueChangedEventArgs e)
        {  
            if(e.Edge == GpioPinEdge.RisingEdge)
            {
                IAsyncAction task = Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
                {
                    this.messageTextBlock.Text = "동작 감지!";
                });
            }

            if(e.Edge == GpioPinEdge.FallingEdge)
            {
                IAsyncAction task = Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
                {
                    this.messageTextBlock.Text = "";
                });
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region GPIO 초기화 하기 - InitializeGPIO()
 
        /// <summary>
        /// GPIO 초기화 하기
        /// </summary>
        private async Task InitializeGPIO()
        {
            this.gpioController = GpioController.GetDefault();

            if(this.gpioController == null)
            {
                this.gpioPin = null;

                return;
            }

            this.gpioPin = gpioController.OpenPin(PIR_PIN_NUMBER);

            if(this.gpioPin.IsDriveModeSupported(GpioPinDriveMode.InputPullUp))
            {
                this.gpioPin.SetDriveMode(GpioPinDriveMode.InputPullUp);
            }
            else
            {
                this.gpioPin.SetDriveMode(GpioPinDriveMode.Input);
            }

            this.messageTextBlock.Text = "PIR 초기화...";

            await Task.Delay(10000);

            this.messageTextBlock.Text = string.Empty;

            this.gpioPin.ValueChanged += gpioPin_ValueChanged;
        }

        #endregion
    }
}